@@ -26,7 +26,7 @@
 #include "gdm-languages.h"
 #include "im.h"
 
-G_DEFINE_TYPE(FcitxImWidget, fcitx_im_widget, GTK_TYPE_HBOX)
+G_DEFINE_TYPE(FcitxImWidget, fcitx_im_widget, GTK_TYPE_VBOX)
 
 enum {
     AVAIL_TREE_IM_STRING,
@@ -46,7 +46,7 @@ typedef struct {
     GHashTable* langTable;
 } foreach_ct;
 
-static void fcitx_im_widget_finalize(GObject* object);
+static void fcitx_im_widget_dispose(GObject* object);
 static void _fcitx_im_widget_connect(FcitxImWidget* self);
 static void _fcitx_im_widget_load(FcitxImWidget* self);
 static void _fcitx_inputmethod_insert_foreach_cb(gpointer data, gpointer user_data);
@@ -71,12 +71,13 @@ static void
 fcitx_im_widget_class_init(FcitxImWidgetClass *klass)
 {
     GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
-    gobject_class->finalize = fcitx_im_widget_finalize;
+    gobject_class->dispose = fcitx_im_widget_dispose;
 }
 
 static void
 fcitx_im_widget_init(FcitxImWidget* self)
 {
+    GtkWidget* hbox = gtk_hbox_new(FALSE, 0);
     self->availimstore = gtk_tree_store_new(AVAIL_N_COLUMNS, G_TYPE_STRING, G_TYPE_POINTER, G_TYPE_STRING);
     self->filtermodel = gtk_tree_model_filter_new(GTK_TREE_MODEL(self->availimstore), NULL);
 
@@ -123,7 +124,7 @@ fcitx_im_widget_init(FcitxImWidget* self)
     gtk_container_add(GTK_CONTAINER(scrolledwindow), self->availimview);
     gtk_box_pack_start(GTK_BOX(vbox), scrolledwindow, TRUE, TRUE, 5);
     gtk_box_pack_start(GTK_BOX(vbox), self->onlycurlangcheckbox, FALSE, TRUE, 5);
-    gtk_box_pack_start(GTK_BOX(self), vbox, TRUE, TRUE, 5);
+    gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 5);
 
     vbox = gtk_vbox_new(FALSE, 0);
 
@@ -140,7 +141,7 @@ fcitx_im_widget_init(FcitxImWidget* self)
     gtk_box_pack_start(GTK_BOX(vbox), self->delimbutton, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(vbox), gtk_label_new(""), TRUE, TRUE, 0);
 
-    gtk_box_pack_start(GTK_BOX(self), vbox, FALSE, TRUE, 5);
+    gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, TRUE, 5);
 
     label = gtk_label_new(_("Current Input Method"));
 
@@ -166,7 +167,7 @@ fcitx_im_widget_init(FcitxImWidget* self)
     gtk_container_add(GTK_CONTAINER(scrolledwindow), self->imview);
     gtk_box_pack_start(GTK_BOX(vbox), scrolledwindow, TRUE, TRUE, 5);
 
-    gtk_box_pack_start(GTK_BOX(self), GTK_WIDGET(vbox), TRUE, TRUE, 5);
+    gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(vbox), TRUE, TRUE, 5);
 
     vbox = gtk_vbox_new(FALSE, 0);
 
@@ -183,7 +184,22 @@ fcitx_im_widget_init(FcitxImWidget* self)
     gtk_box_pack_start(GTK_BOX(vbox), self->movedownbutton, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(vbox), gtk_label_new(""), TRUE, TRUE, 0);
 
-    gtk_box_pack_start(GTK_BOX(self), vbox, FALSE, TRUE, 5);
+    gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, TRUE, 5);
+
+    label = gtk_label_new(NULL);
+    gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
+    gtk_label_set_markup(GTK_LABEL(label),
+                         _("The first input method will be inactive state. Usually you need to put "
+                         "<b>Keyboard</b> or <b>Keyboard - <i>layout name</i></b> in the first place."));
+    GtkWidget* image = gtk_image_new_from_stock(GTK_STOCK_ABOUT, GTK_ICON_SIZE_BUTTON);
+    GtkWidget* infohbox = gtk_hbox_new(FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(infohbox), gtk_label_new(NULL), TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(infohbox), image, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(infohbox), label, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(infohbox), gtk_label_new(NULL), TRUE, TRUE, 0);
+
+    gtk_box_pack_start(GTK_BOX(self), hbox, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(self), infohbox, FALSE, TRUE, 0);
 
     g_signal_connect(G_OBJECT(self->addimbutton), "clicked", G_CALLBACK(_fcitx_im_widget_addim_button_clicked), self);
     g_signal_connect(G_OBJECT(self->delimbutton), "clicked", G_CALLBACK(_fcitx_im_widget_delim_button_clicked), self);
@@ -207,7 +223,7 @@ fcitx_im_widget_new(void)
     return GTK_WIDGET(widget);
 }
 
-void fcitx_im_widget_finalize(GObject* object)
+void fcitx_im_widget_dispose(GObject* object)
 {
     FcitxImWidget* self = FCITX_IM_WIDGET(object);
     if (self->array) {
@@ -215,10 +231,18 @@ void fcitx_im_widget_finalize(GObject* object)
         g_ptr_array_free(self->array, FALSE);
         self->array = NULL;
     }
-    g_object_unref(self->improxy);
-    g_free(self->focus);
 
-    G_OBJECT_CLASS (fcitx_im_widget_parent_class)->finalize (object);
+    if (self->improxy) {
+        g_object_unref(self->improxy);
+        self->improxy = NULL;
+    }
+
+    if (self->focus) {
+        g_free(self->focus);
+        self->focus = NULL;
+    }
+
+    G_OBJECT_CLASS (fcitx_im_widget_parent_class)->dispose (object);
 }
 
 
@@ -521,12 +545,14 @@ gboolean _fcitx_im_widget_filter_func(GtkTreeModel *model,
 
     gboolean flag = TRUE;
     if (item) {
-        flag = flag && (strlen(filter_text) == 0
-                 || strstr(item->name, filter_text)
-                 || strstr(item->unique_name, filter_text)
-                 || strstr(item->langcode, filter_text));
-        flag = flag && (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(self->onlycurlangcheckbox)) ?
-                 strncmp(item->langcode, _get_current_lang() , 2) == 0 : TRUE) ;
+        if (strcmp(item->unique_name, "fcitx-keyboard-us") != 0) {
+            flag = flag && (strlen(filter_text) == 0
+                     || strstr(item->name, filter_text)
+                     || strstr(item->unique_name, filter_text)
+                     || strstr(item->langcode, filter_text));
+            flag = flag && (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(self->onlycurlangcheckbox)) ?
+                     strncmp(item->langcode, _get_current_lang() , 2) == 0 : TRUE) ;
+        }
         return flag;
     } else {
         gchar* lang = NULL;
@@ -534,6 +560,8 @@ gboolean _fcitx_im_widget_filter_func(GtkTreeModel *model,
                            iter,
                            AVAIL_TREE_LANG, &lang,
                            -1);
+        if (lang && strcmp(lang, "en") == 0)
+            return TRUE;
         flag = (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(self->onlycurlangcheckbox)) ?
                 lang != NULL && strncmp(lang, _get_current_lang() , 2) == 0 : TRUE) ;
         g_free(lang);
@@ -560,4 +588,4 @@ icon_press_cb (GtkEntry       *entry,
                gpointer        data)
 {
     gtk_entry_set_text (entry, "");
-}
\ No newline at end of file
+}
